/*
* AllBinary Open License Version 1
* Copyright (c) 2011 AllBinary
* 
* By agreeing to this license you and any business entity you represent are
* legally bound to the AllBinary Open License Version 1 legal agreement.
* 
* You may obtain the AllBinary Open License Version 1 legal agreement from
* AllBinary or the root directory of AllBinary's AllBinary Platform repository.
* 
* Created By: Travis Berthelot
* 
*/
package allbinary.logic.visual.transform.template.customizer.widgets.logo;

import abcs.data.tree.dom.DomSearchHelper;
import abcs.logic.basic.io.file.FileData;
import abcs.logic.basic.path.AbPath;
import abcs.logic.basic.string.StringValidationUtil;
import abcs.logic.communication.log.LogFactory;
import abcs.logic.communication.log.LogUtil;
import allbinary.data.tree.dom.DomNodeInterface;
import allbinary.data.tree.dom.ModDomHelper;
import allbinary.logic.communication.http.request.HttpRequestUtil;
import allbinary.logic.control.validate.ValidationInterface;
import org.apache.commons.fileupload.FileItem;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import java.util.HashMap;

public class LogoValidation implements ValidationInterface, DomNodeInterface
{

    private FileItem logoFileItem;
    private AbPath logoAbPath;
    private String logoFile;
    private boolean isFile;

    public LogoValidation()
    {
        this.isFile = false;

        this.logoAbPath = null;
        this.logoFile = null;
    }

    public LogoValidation(Node node) throws Exception
    {
        this.isFile = false;

        this.logoAbPath = new AbPath(DomSearchHelper.getNode(
            LogoData.getInstance().IMAGEPATH, node.getChildNodes()).getNodeValue());

        this.logoFile = DomSearchHelper.getNode(
            LogoData.getInstance().IMAGEFILENAME, node.getChildNodes()).getNodeValue();
    }

    public LogoValidation(HashMap hashMap) throws Exception
    {
        this.isFile = true;

        this.getFormData(hashMap);
    }

    public void getFormData(HashMap hashMap) throws Exception
    {
        this.logoFileItem = (FileItem) hashMap.get(LogoData.getInstance().IMAGE);
        this.logoAbPath = new AbPath((String) hashMap.get(LogoData.getInstance().IMAGEPATH));

        if (this.logoFileItem != null)
        {
            this.logoFile = HttpRequestUtil.getInstance().generateFileName(
            		this.logoFileItem.getName());
        }

        if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
        {
            StringBuffer stringBuffer = new StringBuffer();

            stringBuffer.append("HashMap: ");
            stringBuffer.append(hashMap.toString());
            stringBuffer.append("\nLogoFileName: ");
            stringBuffer.append(this.logoFile);

            LogUtil.put(LogFactory.getInstance(stringBuffer.toString(), this, "getFormData()"));
        }
    }

    public Boolean isValid()
    {
        try
        {
            Boolean valid = Boolean.TRUE;

            FileData fileData = FileData.getInstance();
            
            if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
            {
                LogUtil.put(LogFactory.getInstance("LogoValidation", this, "isValid()"));
            }

            if (!StringValidationUtil.getInstance().isValidRequired(this.logoAbPath.toString(), 0, 512))
            {
                if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
                {
                    LogUtil.put(LogFactory.getInstance("logoPath is invalid: " + this.logoAbPath.toString(), this, "isValid()"));
                }
                return Boolean.FALSE;
            }

            if (!StringValidationUtil.getInstance().isValidRequired(
                this.logoFile, fileData.MINLEN, fileData.MAXLEN))
            {
                if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
                {
                    LogUtil.put(LogFactory.getInstance("logoFile is invalid: " + this.logoFile, this, "isValid()"));
                }
                return Boolean.FALSE;
            }

            //future version may require file for edit
            if (this.isFile)
            {
                if (this.logoFileItem == null)
                {
                    return Boolean.FALSE;
                }

                if (this.logoFileItem.getSize() > fileData.MAXIMAGEFILESIZE || logoFileItem.getSize() < fileData.MINIMAGEFILESIZE)
                {
                    if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
                    {
                        LogUtil.put(LogFactory.getInstance("Image File Is Not The Right Size. "
                            + fileData.MINIMAGEFILESIZE + "< > "
                            + fileData.MAXIMAGEFILESIZE, this, "isValid()"));
                    }
                    return Boolean.FALSE;
                }
            }

            if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
            {
                LogUtil.put(LogFactory.getInstance("Logo Validation: " + valid, this, "isValid()"));
            }

            return valid;
        } catch (Exception e)
        {
            if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.SQLTAGSERROR))
            {
                LogUtil.put(LogFactory.getInstance("Failed to validate form", this, "isValid()", e));
            }
            return Boolean.FALSE;
        }
    }

    public String validationInfo()
    {
        try
        {
            StringBuffer stringBuffer = new StringBuffer();

            stringBuffer.append("Logo is not valid.");

            return stringBuffer.toString();
        } catch (Exception e)
        {
            if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.SQLTAGSERROR))
            {
                LogUtil.put(LogFactory.getInstance("Failed to generate validation error info", this, "validationInfo()", e));
            }
            return "Error Validating Form";
        }
    }

    public Document toValidationInfoDoc()
    {
        return null;
    }

    public Node toValidationInfoNode(Document document)
    {
        return null;
    }

    public HashMap toHashMap()
    {
        HashMap hashMap = new HashMap();
        hashMap.put(LogoData.getInstance().IMAGE, "");
        if (this.logoFile != null)
        {
            hashMap.put(LogoData.getInstance().IMAGEPATH, this.logoAbPath.toString());
            hashMap.put(LogoData.getInstance().IMAGEFILENAME, this.logoFile);
        } else
        {
            hashMap.put(LogoData.getInstance().IMAGEPATH, null);
            hashMap.put(LogoData.getInstance().IMAGEFILENAME, this.logoFile);
        }

        return hashMap;
    }

    public Node toXmlNode(Document document) throws Exception
    {
        HashMap hashMap = this.toHashMap();
        return ModDomHelper.createNameValueNodes(document, LogoData.getInstance().NAME, hashMap);
    }

    public void processLogoFile() throws Exception
    {
        if (abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.VIEW))
        {
            StringBuffer stringBuffer = new StringBuffer();

            stringBuffer.append("Processing logoFile: ");
            stringBuffer.append(this.logoAbPath.toString());
            stringBuffer.append(this.logoFile);

            LogUtil.put(LogFactory.getInstance(stringBuffer.toString(), this, "processLogoFile()"));
        }
        //Do not update image if file was not provided
        new LogoImageFileUtil(this.logoAbPath, this.logoFile).saveFiles(this.logoFileItem);
        //if(!imageFile.isFile()) throw new Exception("Image File Did Not Save");
    }
}
