/*
* AllBinary Open License Version 1
* Copyright (c) 2011 AllBinary
* 
* By agreeing to this license you and any business entity you represent are
* legally bound to the AllBinary Open License Version 1 legal agreement.
* 
* You may obtain the AllBinary Open License Version 1 legal agreement from
* AllBinary or the root directory of AllBinary's AllBinary Platform repository.
* 
* Created By: Travis Berthelot
* 
*/
package allbinary.business.user.commerce.money.payment.gateway.transaction;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

import abcs.logic.basic.io.file.generators.OrderIdGenerator;
import abcs.logic.basic.string.StringValidationUtil;
import abcs.logic.communication.log.LogFactory;
import abcs.logic.communication.log.LogUtil;
import allbinary.business.context.modules.storefront.StoreFrontFactory;
import allbinary.business.context.modules.storefront.StoreFrontInterface;
import allbinary.business.user.address.StreetAddress;
import allbinary.business.user.commerce.inventory.basket.Basket;
import allbinary.business.user.commerce.inventory.order.OrderHistory;
import allbinary.business.user.commerce.inventory.order.OrderInterface;
import allbinary.business.user.commerce.money.Money;
import allbinary.business.user.commerce.money.payment.Payment;
import allbinary.business.user.commerce.money.payment.PaymentData;
import allbinary.business.user.commerce.money.payment.gateway.PaymentGatewayData;
import allbinary.business.user.commerce.money.payment.types.TenderTypeFactory;
import allbinary.business.user.commerce.money.payment.types.TransactionTypeFactory;
import allbinary.data.tables.user.commerce.inventory.order.OrderHistoryEntityFactory;
import allbinary.data.tables.user.commerce.inventory.order.OrderHistoryEntityInterface;
import allbinary.logic.communication.http.request.session.WeblisketSession;
import allbinary.logic.visual.transform.info.TransformInfoHttpInterface;
import allbinary.logic.visual.transform.info.TransformInfoInterface;

public class PaymentTransactionInterfaceRequestFactory
{
	private static final PaymentTransactionInterfaceRequestFactory instance = 
		new PaymentTransactionInterfaceRequestFactory();

   private PaymentTransactionInterfaceRequestFactory()
   {
   }
   
   public static PaymentTransactionInterface getInstance(
      TransformInfoInterface transformInfoInterface)
      throws Exception
   {
      try
      {
         TransformInfoHttpInterface httpTransformInfoInterface = 
            (TransformInfoHttpInterface) transformInfoInterface;

         PageContext pageContext = httpTransformInfoInterface.getPageContext();
         
         HttpServletRequest httpServletRequest =
            (HttpServletRequest) pageContext.getRequest();
         
         String command =
            httpServletRequest.getParameter(allbinary.globals.GLOBALS.ADMINCOMMAND);
         
         //Only if testing a gateway with a form instead of using OrderHistory
         if(command!=null && command.compareTo(allbinary.globals.GLOBALS.AUTHORIZEFORMEVALBODYONERROR)==0)
         {
            return generateFromTestData(transformInfoInterface);
         }
         else
         {
            
            HashMap propertiesHashMap = httpTransformInfoInterface.getPropertiesHashMap();

            WeblisketSession weblisketSession =
               new WeblisketSession(propertiesHashMap, pageContext);

            StoreFrontInterface storeFrontInterface =
               StoreFrontFactory.getInstance(weblisketSession.getStoreName());

            OrderInterface order = weblisketSession.getOrder();
            String orderId = order.getId();
            
            OrderHistoryEntityInterface orderHistoryEntityInterface =
               OrderHistoryEntityFactory.getInstance();
            
            OrderHistory orderReview = 
               orderHistoryEntityInterface.getOrder(orderId);
            Payment orderPaymentInfo = orderReview.getPaymentInfo();
            orderPaymentInfo.setTransactionType(TransactionTypeFactory.getInstance().SALE.toString());
            orderPaymentInfo.setTenderType(TenderTypeFactory.getInstance().CREDITCARD.toString());
            orderReview.setPaymentInfo(orderPaymentInfo);
            
            if(abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.PAYMENT))
            {
               LogUtil.put(LogFactory.getInstance(
                       orderReview.getId(), instance, "getPaymentTransactionInterface()"));
            }
            //HashMap hashMap = new HashMap(httpServletRequest.getParameterMap());
            //this.paymentGatewayInterface = new PaymentGateway(hashMap);
            PaymentTransactionInterface paymentTransactionInterface =
               PaymentTransactionInterfaceFactory.getInstance().getInstance(orderReview);
            return paymentTransactionInterface;
         }
         
      }
      catch(Exception e)
      {
         String error = "Error Getting PaymentTransactionInterface";
         if(abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.PAYMENTERROR))
         {
            LogUtil.put(LogFactory.getInstance(
                    error, instance, "getPaymentTransactionInterface()", e));
         }
         throw e;
      }
   }

   private static PaymentTransactionInterface
      generateFromTestData(
      TransformInfoInterface transformInfoInterface)
      throws Exception
   {
      try
      {
         TransformInfoHttpInterface httpTransformInfoInterface = 
            (TransformInfoHttpInterface) transformInfoInterface;
         
         HashMap propertiesHashMap = httpTransformInfoInterface.getPropertiesHashMap();
         PageContext pageContext = httpTransformInfoInterface.getPageContext();
         
         WeblisketSession weblisketSession =
            new WeblisketSession(propertiesHashMap, pageContext);
         
         StoreFrontInterface storeFrontInterface =
            StoreFrontFactory.getInstance(weblisketSession.getStoreName());
         
         HttpServletRequest httpServletRequest =
            (HttpServletRequest) pageContext.getRequest();
         
         String gatewayName =
            (String) weblisketSession.getPaymentMethod();
         
         if(StringValidationUtil.getInstance().isEmpty(gatewayName))
         {
            gatewayName = httpServletRequest.getParameter(
               PaymentGatewayData.NAME.toString());
         }
         
         String transactionType =
            httpServletRequest.getParameter(PaymentData.TRANSACTIONTYPE);
         String tenderType =
            httpServletRequest.getParameter(PaymentData.TENDERTYPE);

         Payment payment = new Payment();
         payment.setTransactionType(transactionType);
         payment.setTenderType(tenderType);

         OrderHistory orderReview = new OrderHistory(new Basket());
         orderReview.setStoreName(storeFrontInterface.getName());
         orderReview.setPaymentMethod(gatewayName);
         orderReview.setUserName("testing");

         String orderId = httpServletRequest.getParameter("orderNumber");
         if(orderId==null || orderId.compareTo("")==0)
         {
            orderId = new OrderIdGenerator().getNext();
            orderReview.setId(orderId);
            //payment information
            String account = httpServletRequest.getParameter("account");
            String expirationDate = httpServletRequest.getParameter("expirationDate");

            String checkNumber = httpServletRequest.getParameter("checkNumber");
            String driversLicense = httpServletRequest.getParameter("driversLicense");

            String magneticInkCheckReader =
               httpServletRequest.getParameter("magneticInkCheckReader");

            String aba = httpServletRequest.getParameter("aba");
            String accountType = httpServletRequest.getParameter("accountType");
            String name = httpServletRequest.getParameter("name");

            payment.setName(name);
            payment.setNumber(account);
            payment.setExpiration(expirationDate);

            payment.setCheckNumber(checkNumber);
            payment.setDriversLicense(driversLicense);
            payment.setMagneticInkCheckReader(magneticInkCheckReader);
            payment.setAba(aba);
            payment.setAccountType(accountType);

            //billing address info
            String street = httpServletRequest.getParameter("street");
            String city = httpServletRequest.getParameter("city");
            String state = httpServletRequest.getParameter("state");
            String zip = httpServletRequest.getParameter("zip");

            StreetAddress streetAddress = new StreetAddress();
            streetAddress.setName(name);
            streetAddress.setStreet(street);
            streetAddress.setCity(city);
            streetAddress.setState(state);
            streetAddress.setCode(zip);

            String email = httpServletRequest.getParameter("email");
            String amount = httpServletRequest.getParameter("amount");

            orderReview.setTotal(new Money(amount));
            orderReview.setBillingAddress(streetAddress);
         }
         else
         {
            orderReview.setId(orderId);
         }
         orderReview.setPaymentInfo(payment);
         
         PaymentTransactionInterface paymentTransactionInterface =
            PaymentTransactionInterfaceFactory.getInstance().getInstance(orderReview);
         return paymentTransactionInterface;
      }
      catch(Exception e)
      {
         String error = "Error Getting PaymentTransactionInterface";
         if(abcs.logic.communication.log.config.type.LogConfigTypes.LOGGING.contains(abcs.logic.communication.log.config.type.LogConfigType.PAYMENTERROR))
         {
            LogUtil.put(LogFactory.getInstance(
                    error, instance, "generatePaymentTransactionInterfaceFromTestData()", e));
         }
         throw e;
      }
   }
}
