/*
* AllBinary Open License Version 1
* Copyright (c) 2011 AllBinary
* 
* By agreeing to this license you and any business entity you represent are
* legally bound to the AllBinary Open License Version 1 legal agreement.
* 
* You may obtain the AllBinary Open License Version 1 legal agreement from
* AllBinary or the root directory of AllBinary's AllBinary Platform repository.
* 
* Created By: Travis Berthelot
* 
*/
package allbinary.game.displayable.canvas;

import javax.microedition.lcdui.Canvas;

import org.allbinary.util.BasicArrayList;

import abcs.logic.basic.string.CommonStrings;
import abcs.logic.communication.log.LogFactory;
import abcs.logic.communication.log.LogUtil;
import allbinary.game.input.GameInputStrings;
import allbinary.game.input.event.GameKeyEvent;
import allbinary.graphics.GPoint;
import allbinary.graphics.Rectangle;
import allbinary.graphics.displayable.MyCanvas;
import allbinary.input.motion.gesture.MotionGestureInput;
import allbinary.input.motion.gesture.TouchMotionGestureFactory;
import allbinary.input.motion.gesture.observer.MotionGestureEvent;
import allbinary.math.RectangleCollisionUtil;
import allbinary.time.TimeDelayHelper;

public class PopupMenuInputProcessor extends BasicMenuInputProcessor
{
    private final int CLICK_DELAY = 120;
    private final TimeDelayHelper clickTimeHelper = new TimeDelayHelper(CLICK_DELAY);

    private Rectangle rectangle;

    public PopupMenuInputProcessor(
        BasicArrayList gameKeyEventList,
        MyCanvas gameCanvas,
        Rectangle rectangle)
    {
        super(gameKeyEventList, gameCanvas);

        this.rectangle = rectangle;
    }

    public void init(Rectangle rectangle)
    {
        this.rectangle = rectangle;        
    }
    
    public int processInput(int key) throws Exception
    {
        //LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().START_LABEL + "Canvas." + CanvasUtil.getKeyName(key), this, GameInputStrings.getInstance()));

        if (key == Canvas.KEY_NUM5)
        {
            //LogUtil.put(LogFactory.getInstance("Key: " + key, this, GameInputStrings.getInstance()));
            //PreLogUtil.put("Key: " + key, this, GameInputStrings.getInstance());
                        
            ((AllBinaryGameCanvas) this.getCanvas()).toggleMenu();
            
            return 1;
        }
        return 0;
    }

    public int processInput()
        throws Exception
    {
        try
        {
            int motionInputsIndex = this.processMotionInputs();

            BasicArrayList list = this.getGameKeyEventList();

            int size = list.size();
            int key = 0;

            GameKeyEvent gameKeyEvent;

            for (int index = 0; index < size; index++)
            {
                gameKeyEvent = (GameKeyEvent) list.objectArray[index];

                if (gameKeyEvent != null)
                {
                    key = gameKeyEvent.getKey();

                    if (this.processInput(key) == 1)
                    {
                        break;
                    }
                }
            }

            this.clear();

            if (size > 0 || motionInputsIndex >= 0)
            {
                return 1;
            }
            else
            {
                return -1;
            }
        }
        catch (Exception e)
        {
            LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().EXCEPTION, this, GameInputStrings.getInstance().PROCESS_INPUT, e));
            return -1;
        }
    }

    //AllBinaryLayerManager layerManager
    public int processMotionInputs()
        throws Exception
    {
        int lastIndex = this.motionGestureEventList.size() - 1;

        if (lastIndex >= 0)
        {
            MotionGestureEvent motionGestureEvent =
                (MotionGestureEvent) this.motionGestureEventList.objectArray[lastIndex];

            this.processMotionInput(motionGestureEvent);
        }

        motionGestureEventList.clear();

        return lastIndex;
    }

    protected void processMotionInput(MotionGestureEvent motionGestureEvent)
        throws Exception
    {
        //LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().START, this, "processMotionInput"));

        if(motionGestureEvent == null)
        {
            LogUtil.put(LogFactory.getInstance("Exception: Bug", this, "processMotionInput"));
            return;
        }
        
        MotionGestureInput motionGestureInput =
            motionGestureEvent.getMotionGesture();

        if (motionGestureInput == TouchMotionGestureFactory.getInstance().RELEASED)
        //||
        //motionGestureInput == TouchMotionGestureFactory.getInstance().PRESSED)
        {
            GPoint point = motionGestureEvent.getCurrentPoint();
            
            //PreLogUtil.put("Rect: " + rectangle.toString() + " " + point.toString(), this, "processMotionInput");
            
            GPoint rectPoint = rectangle.getPoint();
            if (RectangleCollisionUtil.isInside(
                rectPoint.getX(),
                rectPoint.getY(),
                //fudge the edge of the button since touch screen calibration may suck
                rectangle.getMaxX() + 20, rectangle.getMaxY(),
                point.getX(), point.getY()))
            {
                if (this.clickTimeHelper.isTime())
                {
                    //PreLogUtil.put("Toggle Menu: " + motionGestureInput.toString(), this, "processMotionInput");

                    ((AllBinaryGameCanvas) this.getCanvas()).toggleMenu();
                }
            }
        }
    }
}
