/*
* AllBinary Open License Version 1
* Copyright (c) 2011 AllBinary
* 
* By agreeing to this license you and any business entity you represent are
* legally bound to the AllBinary Open License Version 1 legal agreement.
* 
* You may obtain the AllBinary Open License Version 1 legal agreement from
* AllBinary or the root directory of AllBinary's AllBinary Platform repository.
* 
* Created By: Travis Berthelot
* 
*/
package allbinary.media.audio;

import javax.microedition.media.Player;

import abcs.logic.basic.string.CommonStrings;
import abcs.logic.basic.string.StringMaker;
import abcs.logic.communication.log.LogFactory;
import abcs.logic.communication.log.LogUtil;
import allbinary.graphics.canvas.transition.progress.ProgressCanvas;
import allbinary.graphics.canvas.transition.progress.ProgressCanvasFactory;
import allbinary.logic.math.SmallIntegerSingletonFactory;

public class Sounds
{
    private final SoundsFactoryInterface soundsFactoryInterface;

    public Sounds(SoundsFactoryInterface soundsFactoryInterface)
    {
        this.soundsFactoryInterface = soundsFactoryInterface;
    }

    public void init() throws Exception
    {
        LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().START, this, CommonStrings.getInstance().INIT));

        CommonStrings commonStrings = CommonStrings.getInstance();
        final String SOUND = " Sound: ";
        
        ProgressCanvas progressCanvas = 
            ProgressCanvasFactory.getInstance();
        
        soundsFactoryInterface.init();

        Sound[] soundInterfaceArray = soundsFactoryInterface.getSoundInterfaceArray();

        final String INIT_SOUND = "Initializing Sound: ";

        final SmallIntegerSingletonFactory smallIntegerSingletonFactory = SmallIntegerSingletonFactory.getInstance();
        
        for (int i = 0; i < soundInterfaceArray.length; i++)
        {

            final Sound soundInterface = soundInterfaceArray[i];

            if (soundInterface != null)
            {
                Integer indexInteger = smallIntegerSingletonFactory.getInstance(i);
                String indexString = indexInteger.toString();
                
                StringMaker stringBuffer = new StringMaker();
                
                stringBuffer.append(commonStrings.INDEX_LABEL);
                stringBuffer.append(indexString);
                stringBuffer.append(SOUND);
                stringBuffer.append(soundInterface.getClass().getName());

                LogUtil.put(LogFactory.getInstance(stringBuffer.toString(), this, CommonStrings.getInstance().INIT));
                
                soundInterface.init();
                
                progressCanvas.addPortion(100, INIT_SOUND + indexString);
            }
        }

        soundsFactoryInterface.setInitialized(true);
        
        LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().END, this, CommonStrings.getInstance().INIT));
    }

    public void stopAll() throws Exception
    {
        LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().START, this, "stopAll"));

        Sound[] soundInterfaceArray = 
            soundsFactoryInterface.getSoundInterfaceArray();

        ProgressCanvas progressCanvas =
            ProgressCanvasFactory.getInstance();
        
        for (int index = 0; index < soundInterfaceArray.length; index++)
        {
            if (soundInterfaceArray[index] != null)
            {
                Player player = soundInterfaceArray[index].getPlayer();

                if (player != null)
                {
                    player.stop();

                    progressCanvas.addPortion(100, "Stopping Sound: ", index);
                }
            }
        }

    }

    public void closeAll() throws Exception
    {
        LogUtil.put(LogFactory.getInstance(CommonStrings.getInstance().START, this, "closeAll"));

        Sound[] soundInterfaceArray = soundsFactoryInterface.getSoundInterfaceArray();

        ProgressCanvas progressCanvas =
            ProgressCanvasFactory.getInstance();

        for (int index = 0; index < soundInterfaceArray.length; index++)
        {
            if (soundInterfaceArray[index] != null)
            {
                Player player = soundInterfaceArray[index].getPlayer();
                if(player != null)
                {
                    player.close();

                    progressCanvas.addPortion(100, "Closing Sound: ", index);
                }
            }
        }
    }
}
